/* (C) 2022 YiRing, Inc. */
package com.yiring.common.web;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.IdUtil;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import com.yiring.common.core.Minio;
import com.yiring.common.core.Result;
import com.yiring.common.core.Status;
import com.yiring.common.param.DownloadParam;
import com.yiring.common.service.UploadProcessService;
import com.yiring.common.util.FileUtils;
import com.yiring.common.vo.ImageInfo;
import io.minio.GetObjectResponse;
import io.minio.StatObjectResponse;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import java.sql.Timestamp;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotBlank;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

/**
 * Minio S3
 *
 * @author Jim
 */

@Slf4j
@Validated
@SuppressWarnings({ "deprecation" })
@ApiSupport(order = -98)
@Api(tags = "文件管理", description = "file")
@RequiredArgsConstructor
@RestController
@RequestMapping("/common/file/")
public class MinioController {

    final Minio minio;
    final UploadProcessService service;

    /**
     * minio 上传文件，成功返回文件 url
     */
    @ApiOperation(value = "文件上传", consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    @PostMapping(value = "upload", headers = HttpHeaders.CONTENT_TYPE + "=" + MediaType.MULTIPART_FORM_DATA_VALUE)
    public Result<String> upload(@ApiParam(value = "文件", required = true) @RequestPart("file") MultipartFile file) {
        try {
            // 获取文件名信息
            String filename = file.getOriginalFilename();
            if (filename == null) {
                throw Status.BAD_REQUEST.exception();
            }

            // 获取文件信息以及默认存储地址
            String uuid = IdUtil.fastSimpleUUID();
            String object = minio.buildUploadPath(filename, "", uuid);

            // 预处理（默认不做任何处理，具体逻辑需自行在外部实现）
            object = service.handle(object, file.getInputStream());

            // 上传原文件
            minio.putObject(file.getInputStream(), file.getContentType(), object);
            return Result.ok(minio.getDefaultURI(object));
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw Status.BAD_REQUEST.exception();
        }
    }

    @ApiOperation(value = "Base64 图片上传")
    @ApiImplicitParam(
        name = "base64Image",
        value = "Base64 图片信息",
        example = "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAQAAAALCAYAAABYpyyrAAAAAXNSR0IArs4c6QAAAARnQU1BAACxjwv8YQUAAAAJcEhZcwAAFiUAABYlAUlSJPAAAAATSURBVBhXYzAwNP6PjIeKgPF/ABj+RUX4hZfVAAAAAElFTkSuQmCC",
        required = true,
        paramType = "query"
    )
    @PostMapping(value = "uploadBase64Image")
    public Result<String> uploadBase64Image(@NotBlank(message = "图片 Base64 信息不能为空") String base64Image) {
        try {
            // 解析 Base64 图片信息
            ImageInfo image = FileUtils.parseBase64ImageText(base64Image);

            // 获取文件信息以及默认存储地址
            String uuid = IdUtil.getSnowflakeNextIdStr();
            String object = minio.buildUploadPath(
                uuid,
                "." + image.getWidth() + "x" + image.getHeight() + "." + image.getSuffix()
            );

            // 上传原文件
            minio.putObject(image.getStream(), image.getContentType(), object);
            return Result.ok(minio.getDefaultURI(object));
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw Status.BAD_REQUEST.exception();
        }
    }

    /**
     * MinIO 文件下载（非公开桶）
     *
     * @param response HttpServletResponse
     * @param param    请求参数
     */
    @ApiOperation(value = "文件下载", produces = MediaType.APPLICATION_OCTET_STREAM_VALUE)
    @GetMapping("download")
    public void download(HttpServletResponse response, @Validated DownloadParam param) {
        try {
            StatObjectResponse statObject = minio.objectStat(param.getBucket(), param.getObject());
            GetObjectResponse object = minio.getObject(param.getBucket(), param.getObject());
            long lastModified = Timestamp.from(statObject.lastModified().toInstant()).getTime();
            FileUtils.download(
                response,
                object,
                statObject.size(),
                FileUtil.getName(statObject.object()),
                statObject.contentType(),
                lastModified
            );
        } catch (Exception e) {
            throw Status.BAD_REQUEST.exception(e.getMessage());
        }
    }
}
